@@ -17,14 +17,6 @@
 
 package org.apache.doris.spark.client;
 
-import com.fasterxml.jackson.core.JsonProcessingException;
-import com.fasterxml.jackson.databind.JsonNode;
-import com.fasterxml.jackson.databind.ObjectMapper;
-import com.fasterxml.jackson.databind.json.JsonMapper;
-import com.fasterxml.jackson.databind.node.ArrayNode;
-import com.google.common.collect.ImmutableMap;
-import org.apache.commons.lang3.StringUtils;
-import org.apache.commons.lang3.tuple.Pair;
 import org.apache.doris.spark.client.entity.Backend;
 import org.apache.doris.spark.client.entity.Frontend;
 import org.apache.doris.spark.config.DorisConfig;
@@ -36,6 +28,14 @@ import org.apache.doris.spark.rest.models.QueryPlan;
 import org.apache.doris.spark.rest.models.Schema;
 import org.apache.doris.spark.util.HttpUtils;
 import org.apache.doris.spark.util.URLs;
+
+import com.fasterxml.jackson.databind.JsonNode;
+import com.fasterxml.jackson.databind.ObjectMapper;
+import com.fasterxml.jackson.databind.json.JsonMapper;
+import com.fasterxml.jackson.databind.node.ArrayNode;
+import com.google.common.collect.ImmutableMap;
+import org.apache.commons.lang3.StringUtils;
+import org.apache.commons.lang3.tuple.Pair;
 import org.apache.http.HttpResponse;
 import org.apache.http.HttpStatus;
 import org.apache.http.client.methods.CloseableHttpResponse;
@@ -49,6 +49,7 @@ import org.slf4j.LoggerFactory;
 
 import java.io.IOException;
 import java.io.Serializable;
+import java.nio.charset.StandardCharsets;
 import java.sql.Connection;
 import java.sql.DriverManager;
 import java.sql.PreparedStatement;
@@ -101,20 +102,17 @@ public class DorisFrontendClient implements Serializable {
             for (String frontendNode : frontendNodeArray) {
                 String[] nodeDetails = frontendNode.split(":");
                 try {
-                    List<Frontend> list = Collections.singletonList(new Frontend(nodeDetails[0], Integer.parseInt(nodeDetails[1])));
+                    List<Frontend> list = Collections.singletonList(new Frontend(nodeDetails[0],
+                            nodeDetails.length > 1 ? Integer.parseInt(nodeDetails[1]) : -1));
                     frontendList = requestFrontends(list, (frontend, client) -> {
-                        HttpGet httpGet = new HttpGet(URLs.getFrontEndNodes(frontend.getHost(), frontend.getHttpPort(), isHttpsEnabled));
+                        String url = URLs.getFrontEndNodes(frontend.getHost(), frontend.getHttpPort(),
+                                isHttpsEnabled);
+                        HttpGet httpGet = new HttpGet(url);
                         HttpUtils.setAuth(httpGet, username, password);
                         JsonNode dataNode;
                         try {
                             HttpResponse response = client.execute(httpGet);
-                            if (response.getStatusLine().getStatusCode() != HttpStatus.SC_OK) {
-                                throw new RuntimeException("fetch fe request failed, status: "
-                                        + response.getStatusLine().getStatusCode()
-                                        + ", reason: " + response.getStatusLine().getReasonPhrase());
-                            }
-                            String entity = EntityUtils.toString(response.getEntity());
-                            dataNode = extractEntity(entity, "data");
+                            dataNode = extractDataFromResponse(response, url);
                         } catch (IOException e) {
                             throw new RuntimeException("fetch fe failed", e);
                         }
@@ -142,7 +140,7 @@ public class DorisFrontendClient implements Serializable {
             return Arrays.stream(frontendNodeArray)
                     .map(node -> {
                         String[] nodeParts = node.split(":");
-                        return new Frontend(nodeParts[0], Integer.parseInt(nodeParts[1]), queryPort, flightSqlPort);
+                        return new Frontend(nodeParts[0], nodeParts.length > 1 ? Integer.parseInt(nodeParts[1]) : -1, queryPort, flightSqlPort);
                     })
                     .collect(Collectors.toList());
         }
@@ -248,17 +246,14 @@ public class DorisFrontendClient implements Serializable {
 
     public Schema getTableSchema(String db, String table) throws Exception {
         return requestFrontends((frontend, httpClient) -> {
-            HttpGet httpGet = new HttpGet(URLs.tableSchema(frontend.getHost(), frontend.getHttpPort(), db, table, isHttpsEnabled));
+            String url = URLs.tableSchema(frontend.getHost(), frontend.getHttpPort(), db, table, isHttpsEnabled);
+            HttpGet httpGet = new HttpGet(url);
             HttpUtils.setAuth(httpGet, username, password);
             Schema dorisSchema;
             try {
                 HttpResponse response = httpClient.execute(httpGet);
-                if (response.getStatusLine().getStatusCode() != HttpStatus.SC_OK) {
-                    throw new RuntimeException("table schema request failed, code: " + response.getStatusLine().getStatusCode()
-                            + ", reason: " + response.getStatusLine().getReasonPhrase());
-                }
-                String entity = EntityUtils.toString(response.getEntity());
-                dorisSchema = MAPPER.readValue(extractEntity(entity, "data").traverse(), Schema.class);
+                JsonNode dataNode = extractDataFromResponse(response, url);
+                dorisSchema = MAPPER.readValue(dataNode.traverse(), Schema.class);
             } catch (IOException e) {
                 throw new RuntimeException("table schema request failed", e);
             }
@@ -309,17 +304,16 @@ public class DorisFrontendClient implements Serializable {
     public QueryPlan getQueryPlan(String database, String table, String sql) throws Exception {
         return requestFrontends((frontend, httpClient) -> {
             try {
-                HttpPost httpPost = new HttpPost(URLs.queryPlan(frontend.getHost(), frontend.getHttpPort(), database, table, isHttpsEnabled));
+                String url = URLs.queryPlan(frontend.getHost(), frontend.getHttpPort(), database, table, isHttpsEnabled);
+                HttpPost httpPost = new HttpPost(url);
                 HttpUtils.setAuth(httpPost, username, password);
                 String body = MAPPER.writeValueAsString(ImmutableMap.of("sql", sql));
-                httpPost.setEntity(new StringEntity(body));
+                StringEntity stringEntity = new StringEntity(body, StandardCharsets.UTF_8);
+                stringEntity.setContentEncoding("UTF-8");
+                stringEntity.setContentType("application/json");
+                httpPost.setEntity(stringEntity);
                 HttpResponse response = httpClient.execute(httpPost);
-                if (response.getStatusLine().getStatusCode() != HttpStatus.SC_OK) {
-                    throw new DorisException("query plan request failed, code: " + response.getStatusLine().getStatusCode()
-                            + ", reason: " + response.getStatusLine().getReasonPhrase());
-                }
-                String entity = EntityUtils.toString(response.getEntity());
-                JsonNode dataJsonNode = extractEntity(entity, "data");
+                JsonNode dataJsonNode = extractDataFromResponse(response, url);
                 if (dataJsonNode.get("exception") != null) {
                     throw new DorisException("query plan failed, exception: " + dataJsonNode.get("exception").asText());
                 }
@@ -330,8 +324,20 @@ public class DorisFrontendClient implements Serializable {
         });
     }
 
-    private JsonNode extractEntity(String entityStr, String fieldName) throws JsonProcessingException {
-        return MAPPER.readTree(entityStr).get(fieldName);
+
+    private JsonNode extractDataFromResponse(HttpResponse response, String url) throws IOException {
+        if (response.getStatusLine().getStatusCode() != HttpStatus.SC_OK) {
+            throw new RuntimeException("request fe with url: [" + url + "] failed with http code: "
+                    + response.getStatusLine().getStatusCode() + ", reason: "
+                    + response.getStatusLine().getReasonPhrase());
+        }
+        String entity = EntityUtils.toString(response.getEntity());
+        JsonNode respNode = MAPPER.readTree(entity);
+        String code = respNode.get("code").asText();
+        if (!"0".equalsIgnoreCase(code)) {
+            throw new RuntimeException("fetch fe url:[" + url + "] failed with invalid msg code, response: " + entity);
+        }
+        return respNode.get("data");
     }
 
     public String[] getTableAllColumns(String db, String table) throws Exception {
@@ -347,8 +353,8 @@ public class DorisFrontendClient implements Serializable {
             ArrayNode backendsNode;
             try {
                 CloseableHttpResponse res = client.execute(httpGet);
-                String content = EntityUtils.toString(res.getEntity());
-                backendsNode = (ArrayNode) extractEntity(content, "data").get("backends");
+                JsonNode dataNode = extractDataFromResponse(res, url);
+                backendsNode = (ArrayNode) dataNode.get("backends");
             } catch (IOException e) {
                 throw new RuntimeException("get alive backends failed", e);
             }
@@ -362,6 +368,18 @@ public class DorisFrontendClient implements Serializable {
         });
     }
 
+    public void truncateTable(String database, String table) throws Exception {
+        queryFrontends(conn -> {
+            String sql = "TRUNCATE TABLE " + database + "." + table;
+            try (PreparedStatement preparedStatement = conn.prepareStatement(sql)) {
+                preparedStatement.execute();
+                return null;
+            } catch (SQLException e) {
+                throw new RuntimeException("truncate table failed", e);
+            }
+        });
+    }
+
     public List<Frontend> getFrontends() {
         return frontends;
     }
