@@ -17,8 +17,6 @@
 
 package org.apache.doris.spark.client.write;
 
-import com.fasterxml.jackson.databind.DeserializationFeature;
-import com.fasterxml.jackson.databind.json.JsonMapper;
 import org.apache.doris.spark.client.DorisBackendHttpClient;
 import org.apache.doris.spark.client.DorisFrontendClient;
 import org.apache.doris.spark.client.entity.Backend;
@@ -27,8 +25,15 @@ import org.apache.doris.spark.config.DorisConfig;
 import org.apache.doris.spark.config.DorisOptions;
 import org.apache.doris.spark.exception.OptionRequiredException;
 import org.apache.doris.spark.exception.StreamLoadException;
+import org.apache.doris.spark.rest.models.DataFormat;
+import org.apache.doris.spark.util.EscapeHandler;
 import org.apache.doris.spark.util.HttpUtils;
 import org.apache.doris.spark.util.URLs;
+
+import com.fasterxml.jackson.core.JsonProcessingException;
+import com.fasterxml.jackson.databind.DeserializationFeature;
+import com.fasterxml.jackson.databind.ObjectMapper;
+import com.fasterxml.jackson.databind.json.JsonMapper;
 import org.apache.http.HttpEntity;
 import org.apache.http.HttpHeaders;
 import org.apache.http.HttpStatus;
@@ -59,59 +64,44 @@ import java.util.concurrent.Future;
 import java.util.concurrent.atomic.AtomicInteger;
 import java.util.stream.Collectors;
 
-public abstract class AbstractStreamLoadProcessor<R> implements DorisWriter<R>, DorisCommitter {
-
-    protected final Logger logger = LoggerFactory.getLogger(this.getClass().getName().replaceAll("\\$", ""));
-
-    protected static final JsonMapper MAPPER = JsonMapper.builder().configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false).build();
+public abstract class AbstractStreamLoadProcessor<R> extends DorisWriter<R> implements DorisCommitter {
 
+    protected static final JsonMapper MAPPER =
+            JsonMapper.builder().configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false).build();
     private static final String PARTIAL_COLUMNS = "partial_columns";
     private static final String GROUP_COMMIT = "group_commit";
-    private static final Set<String> VALID_GROUP_MODE = new HashSet<>(Arrays.asList("sync_mode", "async_mode", "off_mode"));
-
+    private static final Set<String> VALID_GROUP_MODE =
+            new HashSet<>(Arrays.asList("sync_mode", "async_mode", "off_mode"));
+    private static final int arrowBufferSize = 1000;
+    protected final Logger logger = LoggerFactory.getLogger(this.getClass().getName().replaceAll("\\$", ""));
     protected final DorisConfig config;
-
     private final DorisFrontendClient frontend;
     private final DorisBackendHttpClient backendHttpClient;
-
     private final String database;
     private final String table;
-
     private final boolean autoRedirect;
-
     private final boolean isHttpsEnabled;
-
     private final boolean isTwoPhaseCommitEnabled;
-
     private final Map<String, String> properties;
-
-    private final String format;
-
-    protected String columnSeparator;
-
-    private String lineDelimiter;
-
+    private final DataFormat format;
     private final boolean isGzipCompressionEnabled;
-
-    private String groupCommit;
-
     private final boolean isPassThrough;
-
-    private PipedOutputStream output;
-
-    private boolean createNewBatch = true;
-
-    private boolean isFirstRecordOfBatch = true;
-
     private final List<R> recordBuffer = new LinkedList<>();
-
-    private static final int arrowBufferSize = 1000;
-
+    private final int pipeSize;
+    protected String columnSeparator;
+    private byte[] lineDelimiter;
+    private String groupCommit;
+    private PipedOutputStream output;
+    private boolean createNewBatch = true;
+    private boolean isFirstRecordOfBatch = true;
     private transient ExecutorService executor;
 
-    private Future<CloseableHttpResponse> requestFuture = null;
+    private Future<StreamLoadResponse> requestFuture = null;
+    private volatile String currentLabel;
+    private Exception unexpectedException = null;
 
     public AbstractStreamLoadProcessor(DorisConfig config) throws Exception {
+        super(config.getValue(DorisOptions.DORIS_SINK_BATCH_SIZE));
         this.config = config;
         String tableIdentifier = config.getValue(DorisOptions.DORIS_TABLE_IDENTIFIER);
         String[] dbTableArr = tableIdentifier.split("\\.");
@@ -124,23 +114,32 @@ public abstract class AbstractStreamLoadProcessor<R> implements DorisWriter<R>,
         this.properties = config.getSinkProperties();
         // init stream load props
         this.isTwoPhaseCommitEnabled = config.getValue(DorisOptions.DORIS_SINK_ENABLE_2PC);
-        this.format = properties.getOrDefault("format", "csv");
-        this.isGzipCompressionEnabled = properties.containsKey("compress_type") && "gzip".equals(properties.get("compress_type"));
+        this.format = DataFormat.valueOf(properties.getOrDefault("format", "csv").toUpperCase());
+        this.isGzipCompressionEnabled =
+                properties.containsKey("compress_type") && "gzip".equals(properties.get("compress_type"));
         if (properties.containsKey(GROUP_COMMIT)) {
             String message = "";
-            if (!isTwoPhaseCommitEnabled) message = "group commit does not support two-phase commit";
-            if (properties.containsKey(PARTIAL_COLUMNS) && "true".equalsIgnoreCase(properties.get(PARTIAL_COLUMNS)))
+            if (isTwoPhaseCommitEnabled) {
+                message = "group commit does not support two-phase commit";
+            }
+            if (properties.containsKey(PARTIAL_COLUMNS) && "true".equalsIgnoreCase(properties.get(PARTIAL_COLUMNS))) {
                 message = "group commit does not support partial column updates";
-            if (!VALID_GROUP_MODE.contains(properties.get(GROUP_COMMIT).toLowerCase()))
+            }
+            if (!VALID_GROUP_MODE.contains(properties.get(GROUP_COMMIT).toLowerCase())) {
                 message = "Unsupported group commit mode: " + properties.get(GROUP_COMMIT);
-            if (!message.isEmpty()) throw new IllegalArgumentException(message);
+            }
+            if (!message.isEmpty()) {
+                throw new IllegalArgumentException(message);
+            }
             groupCommit = properties.get(GROUP_COMMIT).toLowerCase();
         }
         this.isPassThrough = config.getValue(DorisOptions.DORIS_SINK_STREAMING_PASSTHROUGH);
+        this.pipeSize = config.getValue(DorisOptions.DORIS_SINK_NET_BUFFER_SIZE);
     }
 
     public void load(R row) throws Exception {
         if (createNewBatch) {
+            createNewBatch = false;
             if (autoRedirect) {
                 requestFuture = frontend.requestFrontends((frontEnd, httpClient) ->
                         buildReqAndExec(frontEnd.getHost(), frontEnd.getHttpPort(), httpClient));
@@ -148,33 +147,71 @@ public abstract class AbstractStreamLoadProcessor<R> implements DorisWriter<R>,
                 requestFuture = backendHttpClient.executeReq((backend, httpClient) ->
                         buildReqAndExec(backend.getHost(), backend.getHttpPort(), httpClient));
             }
-            createNewBatch = false;
         }
-        output.write(toFormat(row, format));
+        if (isFirstRecordOfBatch) {
+            isFirstRecordOfBatch = false;
+        } else if (lineDelimiter != null) {
+            writeTo(lineDelimiter);
+        }
+        writeTo(toFormat(row, format));
+        currentBatchCount++;
     }
 
     @Override
     public String stop() throws Exception {
-        // arrow format need to send all buffer data before stop
-        if (!recordBuffer.isEmpty() && "arrow".equalsIgnoreCase(format)) {
-            List<R> rs = new LinkedList<>(recordBuffer);
-            recordBuffer.clear();
-            output.write(toArrowFormat(rs));
-        }
-        output.close();
-        CloseableHttpResponse res = requestFuture.get();
-        if (res.getStatusLine().getStatusCode() != HttpStatus.SC_OK) {
-            throw new StreamLoadException("stream load execute failed, status: " + res.getStatusLine().getStatusCode()
-                    + ", msg: " + res.getStatusLine().getReasonPhrase());
-        }
-        String resEntity = EntityUtils.toString(new BufferedHttpEntity(res.getEntity()));
-        logger.info("stream load response: {}", resEntity);
-        StreamLoadResponse response = MAPPER.readValue(resEntity, StreamLoadResponse.class);
-        if (response != null && response.isSuccess()) {
+        if (requestFuture != null) {
             createNewBatch = true;
+            isFirstRecordOfBatch = true;
+            // arrow format need to send all buffer data before stop
+            if (!recordBuffer.isEmpty() && DataFormat.ARROW.equals(format)) {
+                List<R> rs = new LinkedList<>(recordBuffer);
+                recordBuffer.clear();
+                writeTo(toArrowFormat(rs));
+            }
+            output.close();
+            logger.info("stream load stopped with {}", currentLabel != null ? currentLabel : "group commit");
+
+            StreamLoadResponse response;
+            try {
+                response = requestFuture.get();
+                if (response == null) {
+                    throw new StreamLoadException("response is null");
+                }
+            } catch (Exception e) {
+                if (unexpectedException != null) {
+                    throw unexpectedException;
+                }
+                throw new StreamLoadException("stream load stop failed", e);
+            }
             return isTwoPhaseCommitEnabled ? String.valueOf(response.getTxnId()) : null;
-        } else {
-            throw new StreamLoadException("stream load execute failed, response: " + resEntity);
+        }
+        return null;
+    }
+
+    private void execCommitReq(String host, int port, String msg, CloseableHttpClient httpClient) {
+        HttpPut httpPut = new HttpPut(URLs.streamLoad2PC(host, port, database, isHttpsEnabled));
+        try {
+            handleCommitHeaders(httpPut, msg);
+        } catch (OptionRequiredException e) {
+            throw new RuntimeException("stream load handle commit props failed", e);
+        }
+        try {
+            CloseableHttpResponse response = httpClient.execute(httpPut);
+            if (response.getStatusLine().getStatusCode() != HttpStatus.SC_OK) {
+                throw new RuntimeException("commit transaction failed, transaction: " + msg + ", status: "
+                        + response.getStatusLine().getStatusCode() + ", reason: " + response.getStatusLine()
+                        .getReasonPhrase());
+            } else {
+                String resEntity = EntityUtils.toString(new BufferedHttpEntity(response.getEntity()));
+                if (!checkTransResponse(resEntity)) {
+                    throw new RuntimeException(
+                            "commit transaction failed, transaction: " + msg + ", resp: " + resEntity);
+                } else {
+                    this.logger.info("commit: {} response: {}", msg, resEntity);
+                }
+            }
+        } catch (IOException e) {
+            throw new RuntimeException("commit transaction failed, transaction: " + msg, e);
         }
     }
 
@@ -182,62 +219,86 @@ public abstract class AbstractStreamLoadProcessor<R> implements DorisWriter<R>,
     public void commit(String msg) throws Exception {
         if (isTwoPhaseCommitEnabled) {
             logger.info("begin to commit transaction {}", msg);
-            frontend.requestFrontends((frontEnd, httpClient) -> {
-                HttpPut httpPut = new HttpPut(URLs.streamLoad2PC(frontEnd.getHost(), frontEnd.getHttpPort(), database, isHttpsEnabled));
-                try {
-                    handleCommitHeaders(httpPut, msg);
-                } catch (OptionRequiredException e) {
-                    throw new RuntimeException("stream load handle commit props failed", e);
-                }
-                try {
-                    CloseableHttpResponse response = httpClient.execute(httpPut);
-                    if (response.getStatusLine().getStatusCode() != HttpStatus.SC_OK) {
-                        throw new RuntimeException("commit transaction failed, transaction: " + msg
-                                + ", status: " + response.getStatusLine().getStatusCode()
-                                + ", reason: " + response.getStatusLine().getReasonPhrase());
-                    }
-                } catch (IOException e) {
-                    throw new RuntimeException("commit transaction failed, transaction: " + msg, e);
-                }
-                return null;
-            });
+            if (autoRedirect) {
+                frontend.requestFrontends((frontEnd, httpClient) -> {
+                    execCommitReq(frontEnd.getHost(), frontEnd.getHttpPort(), msg, httpClient);
+                    return null;
+                });
+            } else {
+                backendHttpClient.executeReq((backend, httpClient) -> {
+                    execCommitReq(backend.getHost(), backend.getHttpPort(), msg, httpClient);
+                    return null;
+                });
+            }
             logger.info("success to commit transaction {}", msg);
         }
     }
 
+    private void execAbortReq(String host, int port, String msg, CloseableHttpClient httpClient) {
+        HttpPut httpPut = new HttpPut(URLs.streamLoad2PC(host, port, database, isHttpsEnabled));
+        try {
+            handleAbortHeaders(httpPut, msg);
+        } catch (OptionRequiredException e) {
+            throw new RuntimeException("stream load handle abort props failed", e);
+        }
+        try {
+            CloseableHttpResponse response = httpClient.execute(httpPut);
+            if (response.getStatusLine().getStatusCode() != HttpStatus.SC_OK) {
+                throw new RuntimeException("abort transaction failed, transaction: " + msg + ", status: "
+                        + response.getStatusLine().getStatusCode() + ", reason: " + response.getStatusLine()
+                        .getReasonPhrase());
+            } else {
+                String resEntity = EntityUtils.toString(new BufferedHttpEntity(response.getEntity()));
+                if (!checkTransResponse(resEntity)) {
+                    throw new RuntimeException(
+                            "abort transaction failed, transaction: " + msg + ", resp: " + resEntity);
+                } else {
+                    this.logger.info("abort: {} response: {}", msg, resEntity);
+                }
+            }
+        } catch (IOException e) {
+            throw new RuntimeException("abort transaction failed, transaction: " + msg, e);
+        }
+    }
+
+    private boolean checkTransResponse(String resEntity) {
+        ObjectMapper objectMapper = new ObjectMapper();
+        try {
+            String status = objectMapper.readTree(resEntity).get("status").asText();
+            if ("Success".equalsIgnoreCase(status)) {
+                return true;
+            }
+        } catch (JsonProcessingException e) {
+            logger.warn("invalid json response: " + resEntity, e);
+        }
+        return false;
+    }
+
     @Override
     public void abort(String msg) throws Exception {
         if (isTwoPhaseCommitEnabled) {
             logger.info("begin to abort transaction {}", msg);
-            frontend.requestFrontends((frontEnd, httpClient) -> {
-                HttpPut httpPut = new HttpPut(URLs.streamLoad2PC(frontEnd.getHost(), frontEnd.getHttpPort(), database, isHttpsEnabled));
-                try {
-                    handleAbortHeaders(httpPut, msg);
-                } catch (OptionRequiredException e) {
-                    throw new RuntimeException("stream load handle abort props failed", e);
-                }
-                try {
-                    CloseableHttpResponse response = httpClient.execute(httpPut);
-                    if (response.getStatusLine().getStatusCode() != HttpStatus.SC_OK) {
-                        throw new RuntimeException("abort transaction failed, transaction: " + msg
-                                + ", status: " + response.getStatusLine().getStatusCode()
-                                + ", reason: " + response.getStatusLine().getReasonPhrase());
-                    }
-                } catch (IOException e) {
-                    throw new RuntimeException("abort transaction failed, transaction: " + msg, e);
-                }
-                return null; // Returning null as the callback does not return anything
-            });
+            if (autoRedirect) {
+                frontend.requestFrontends((frontEnd, httpClient) -> {
+                    execAbortReq(frontEnd.getHost(), frontEnd.getHttpPort(), msg, httpClient);
+                    return null; // Returning null as the callback does not return anything
+                });
+            } else {
+                backendHttpClient.executeReq((backend, httpClient) -> {
+                    execAbortReq(backend.getHost(), backend.getHttpPort(), msg, httpClient);
+                    return null; // Returning null as the callback does not return anything
+                });
+            }
             logger.info("success to abort transaction {}", msg);
         }
     }
 
-    private byte[] toFormat(R row, String format) throws IOException {
-        switch (format.toLowerCase()) {
-            case "csv":
-            case "json":
+    private byte[] toFormat(R row, DataFormat format) throws IOException {
+        switch (format) {
+            case CSV:
+            case JSON:
                 return toStringFormat(row, format);
-            case "arrow":
+            case ARROW:
                 recordBuffer.add(copy(row));
                 if (recordBuffer.size() < arrowBufferSize) {
                     return new byte[0];
@@ -251,16 +312,13 @@ public abstract class AbstractStreamLoadProcessor<R> implements DorisWriter<R>,
         }
     }
 
-    private byte[] toStringFormat(R row, String format) {
-        String prefix = isFirstRecordOfBatch ? "" : lineDelimiter;
-        isFirstRecordOfBatch = false;
-        String stringRow = isPassThrough ? getPassThroughData(row) : stringify(row, format);
-        return (prefix + stringRow).getBytes(StandardCharsets.UTF_8);
+    private byte[] toStringFormat(R row, DataFormat format) {
+        return isPassThrough ? getPassThroughData(row).getBytes(StandardCharsets.UTF_8) : stringify(row, format);
     }
 
     protected abstract String getPassThroughData(R row);
 
-    public abstract String stringify(R row, String format);
+    public abstract byte[] stringify(R row, DataFormat format);
 
     public abstract byte[] toArrowFormat(List<R> rows) throws IOException;
 
@@ -269,34 +327,33 @@ public abstract class AbstractStreamLoadProcessor<R> implements DorisWriter<R>,
     private void handleStreamLoadProperties(HttpPut httpPut) throws OptionRequiredException {
         addCommonHeaders(httpPut);
         if (groupCommit == null || groupCommit.equals("off_mode")) {
-            String label = generateStreamLoadLabel();
-            httpPut.setHeader("label", label);
+            currentLabel = generateStreamLoadLabel();
+            httpPut.setHeader("label", currentLabel);
         }
         String writeFields = getWriteFields();
         httpPut.setHeader("columns", writeFields);
         if (config.contains(DorisOptions.DORIS_MAX_FILTER_RATIO)) {
             httpPut.setHeader("max_filter_ratio", config.getValue(DorisOptions.DORIS_MAX_FILTER_RATIO));
         }
-        if (isTwoPhaseCommitEnabled) httpPut.setHeader("two_phase_commit", "true");
+        if (isTwoPhaseCommitEnabled) {
+            httpPut.setHeader("two_phase_commit", "true");
+        }
 
-        switch (format.toLowerCase()) {
-            case "csv":
-                if (!properties.containsKey("column_separator")) {
-                    properties.put("column_separator", "\t");
-                }
-                columnSeparator = properties.get("column_separator");
-                if (!properties.containsKey("line_delimiter")) {
-                    properties.put("line_delimiter", "\n");
-                }
-                lineDelimiter = properties.get("line_delimiter");
+        switch (format) {
+            case CSV:
+                // Handling hidden delimiters
+                columnSeparator = EscapeHandler.escapeString(properties.getOrDefault("column_separator", "\t"));
+                lineDelimiter = EscapeHandler.escapeString(properties.getOrDefault("line_delimiter", "\n")
+                ).getBytes(StandardCharsets.UTF_8);
                 break;
-            case "json":
-                if (!properties.containsKey("line_delimiter")) {
-                    properties.put("line_delimiter", "\n");
-                }
-                lineDelimiter = properties.get("line_delimiter");
+            case JSON:
+                lineDelimiter = properties.getOrDefault("line_delimiter", "\n").getBytes(
+                        StandardCharsets.UTF_8);
                 properties.put("read_json_by_line", "true");
                 break;
+            case ARROW:
+                lineDelimiter = null;
+                break;
         }
         properties.forEach(httpPut::setHeader);
     }
@@ -323,14 +380,14 @@ public abstract class AbstractStreamLoadProcessor<R> implements DorisWriter<R>,
 
     protected abstract String generateStreamLoadLabel() throws OptionRequiredException;
 
-    private Future<CloseableHttpResponse> buildReqAndExec(String host, Integer port, CloseableHttpClient client) {
+    private Future<StreamLoadResponse> buildReqAndExec(String host, Integer port, CloseableHttpClient client) {
         HttpPut httpPut = new HttpPut(URLs.streamLoad(host, port, database, table, isHttpsEnabled));
         try {
             handleStreamLoadProperties(httpPut);
         } catch (OptionRequiredException e) {
             throw new RuntimeException("stream load handle properties failed", e);
         }
-        PipedInputStream pipedInputStream = new PipedInputStream(4096);
+        PipedInputStream pipedInputStream = new PipedInputStream(pipeSize);
         try {
             output = new PipedOutputStream(pipedInputStream);
         } catch (IOException e) {
@@ -341,13 +398,50 @@ public abstract class AbstractStreamLoadProcessor<R> implements DorisWriter<R>,
             entity = new GzipCompressingEntity(entity);
         }
         httpPut.setEntity(entity);
-        return getExecutors().submit(() -> client.execute(httpPut));
+        Thread currentThread = Thread.currentThread();
+
+        logger.info("table {}.{} stream load started for {} on host {}:{}", database, table,
+                currentLabel != null ? currentLabel : "group commit", host, port);
+        return getExecutors().submit(() -> {
+            StreamLoadResponse streamLoadResponse = null;
+            try (CloseableHttpResponse response = client.execute(httpPut)) {
+                // stream load http request finished unexpectedly
+                if (response.getStatusLine().getStatusCode() != HttpStatus.SC_OK) {
+                    throw new StreamLoadException(
+                            "stream load failed, status: " + response.getStatusLine().getStatusCode()
+                                    + ", reason: " + response.getStatusLine().getReasonPhrase());
+                }
+                String entityStr = EntityUtils.toString(response.getEntity());
+                logger.info("stream load response: " + entityStr);
+                streamLoadResponse = MAPPER.readValue(entityStr, StreamLoadResponse.class);
+                if (streamLoadResponse == null
+                        || streamLoadResponse.getLabel() == null
+                        || streamLoadResponse.getMessage() == null) {
+                    throw new StreamLoadException("stream load failed, response error : " + entityStr);
+                } else if (!streamLoadResponse.isSuccess()) {
+                    throw new StreamLoadException(
+                            "stream load failed, txnId: " + streamLoadResponse.getTxnId()
+                                    + ", status: " + streamLoadResponse.getStatus()
+                                    + ", msg: " + streamLoadResponse.getMessage());
+                }
+            } catch (Exception e) {
+                logger.error("stream load exception", e);
+                unexpectedException = e;
+                currentThread.interrupt();
+            }
+            return streamLoadResponse;
+        });
     }
 
     @Override
     public void close() throws IOException {
         createNewBatch = true;
+        isFirstRecordOfBatch = true;
+        unexpectedException = null;
         frontend.close();
+        if (backendHttpClient != null) {
+            backendHttpClient.close();
+        }
         if (executor != null && !executor.isShutdown()) {
             executor.shutdown();
         }
@@ -380,4 +474,16 @@ public abstract class AbstractStreamLoadProcessor<R> implements DorisWriter<R>,
         return executor;
     }
 
+    private void writeTo(byte[] bytes) throws Exception {
+        try {
+            output.write(bytes);
+        } catch (Exception e) {
+            if (unexpectedException != null) {
+                throw unexpectedException;
+            } else {
+                throw e;
+            }
+        }
+    }
+
 }
